package edu.wpi.cs.dsrg.utils;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import java.lang.reflect.*;
import java.util.Hashtable;

import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;

/**
 * This class is Used to do printing.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */
 
public class ObjectPrinter extends InheritableVisitor {

	int level = -1;
/**
 * The dummy field used as prefix.
 * @since 1.0
 */	
static String DummyFields = "Class";
/**
 * A String containing all the terminal class name: integer, float, boolean.
 * @since 1.0
 */	
static String TerminalClasses = "java.lang.String java.lang.Integer java.lang.Float java.lang.Boolean";
/**
 * The hash table containing all the objects have been visited.
 * @since 1.0
 */	
Hashtable visitedObjects;

/** 
 * To visit a null object.
 * @since 1.0
 */
public Object dispatch() {
    level++;
    System.out.println(indent() + "<empty/>");
    level--;
    return null;
}
/** 
 * To visit any object.
 * @since 1.0
 */
public Object dispatch(Object obj) {
    level++;
    String classname = obj.getClass().getName();
    if (TerminalClasses.indexOf(classname) != -1) { // if that's a terminal class
        System.out.println(indent() + obj.toString());
        level--;
        return null;
    }
    if (classname.startsWith("[L")) { // it's an array.
        for (int i = 0; i < Array.getLength(obj); i++) {
            System.out.println(indent() + "<index_" + i + ">");
            safeVisit(Array.get(obj, i));
            System.out.println(indent() + "</index_" + i + ">");
        }
        level--;
        return null;
    }
    // else do the following.
    System.out.println(indent() + "<" + classname + ">");
    // iterate throw its properties.
    Field fields[] = obj.getClass().getDeclaredFields();
    for (int i = 0; i < fields.length; i++) {
        try {
            safeVisit(fields[i].get(obj));
        } catch (Exception e) {
            //e.printStackTrace(System.err);
        }
    }
    // find all the get methods.
    Method methods[] = obj.getClass().getMethods();
    for (int i = 0; i < methods.length; i++) {
        String name = methods[i].getName();
        if (name.startsWith("get")) { // it's a get method.
            String objectName = name.substring(3);
            // make sure it's not get dummy fields.
            if (DummyFields.indexOf(objectName) != -1)
                continue;
            level++;
            Object result = null;
            try {
                System.out.println(indent() + "<get_" + objectName + ">");
                // invoke the methods to get a list of results back.
                result = methods[i].invoke(obj, new Object[] {
                });
            } catch (Exception e) {
                //e.printStackTrace(System.err);
            }
            safeVisit(result);
            System.out.println(indent() + "</get_" + objectName + ">");
            level--;
        }
    }
    System.out.println(indent() + "</" + classname + ">");
    level--;
    return null;
}
/**
 * Print out the indent space.
 * @return String The indent string.
 * @since 1.0
 */
String indent() {
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < level; i++)
        buf.append("  ");

    return buf.toString();
}
/**
 * Print the given object out.
 * @param o The object will be printed.
 * @since 1.0
 */
public void print(Object o) {
    level = -1;
    this.visitedObjects = new Hashtable();
    this.visitedObjects.put(o, "ok");
    visit(o);
}
/**
 * This method is used to visit the object that never been visited.
 * @param o The object will be visited.
 * @since 1.0
 */
Object safeVisit(Object obj) {
    if (obj == null)
        return visit(obj);
    else
        if (!this.visitedObjects.containsKey(obj)) {
            this.visitedObjects.put(obj, "ok");
            return visit(obj);
        } else {
            System.out.println(indent() + "<visited/>");
            return null;
        }
}
}
