package edu.wpi.cs.dsrg.xmldb.xat.common.expression;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.sqlgeneration.*;

/**
 * This class defines a arithmatic expression.
 * It's going to represent the leaf expressions, e.g.,
 * A String, Double, or a ColumnName.
 * Hits: You can keep an Object property, that maybe objects of
 * String, Double or ColumnName.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0 
 */

public interface TerminalExpression extends Expression {
	/**
	 * The integer representation of the type of the terminal expression.
	 * @since 1.0
	 */
    final static int STRING = 0;
    final static int DOUBLE = 1;
    final static int NAME = 2;
    final static int BOOL = 3;
/**
 * Get the type of this terminalExpression.
 * It could be a String, or a Double, or a ColumnName.
 * @return int The result got as the type of this terminal expression.
 * @since 1.0
 */
int getType();
/**
 * get the value of this expresssion.
 * @return java.lang.Object The value of this expression.
 * @since 1.0
 */
Object getValue();
/**
 * Set the type of this terminal expression.
 * @param type The type of this expression.
 * @since 1.0
 */
void setType(int type);
/**
 * Set the double value for this terminal expression.
 * @param value The double value of this expression.
 * @since 1.0
 */

public void setValue(double value);
/**
 * Set the column name value for this terminal expression.
 * @param value The column name value of this expression.
 * @since 1.0
 */

public void setValue(ColumnName value);
/**
 * Set the string value for this terminal expression.
 * @param value The string value of this expression.
 * @since 1.0
 */

public void setValue(String value);
/**
 * Set the boolean value for this terminal expression.
 * @param value The boolean value of this expression.
 * @since 1.0
 */

public void setValue(boolean value);
}
