package edu.wpi.cs.dsrg.xmldb.xat.common.operator;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import java.util.Vector;

/**
 * Implementation for the Interface that represents column Names List
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0      
 */

public abstract class ColumnNameListImp
    extends XATOperatorImp
    implements ColumnNameList {
    // vector of ColumnName(s)
    protected Vector vColNames;

    /**
     * Assigns node to current operator.
     * @param node is an XAT node
     * @since   1.0          
     */
    public ColumnNameListImp(XATNode node) {

        super(node);
        vColNames = new Vector();

    }
    /**
     * Appends a new column name to the end of column names list.
     * @param newCol is column name.
     * @since   1.0          
     */

    public void addColumnName(String newCol) {
        vColNames.addElement(newCol);
    }
    /** 
    * Deletes a column name from the list
    * @param oldName is column name
    * @since   1.0         
    */
    public void deleteColumnName(String oldName) {
        int iNumCols = vColNames.size();
        for (int i = 0; i < iNumCols; i++) {
            if (oldName.equals(vColNames.elementAt(i))) {
                vColNames.remove(i);
                return;
            }
        }
    }
    /**
     * Returns a list of column names.
     * @return a list column names.
     * @since   1.0          
     */
    public String[] getColumnNames() {
        return (String[]) vColNames.toArray(new String[] {
        });
    }
    /** 
    * Inserts a column name in the list. 
    * @param newName is column name.
    * @param position is insertion poistion.
    * @since   1.0         
    */

    public void insertColumnName(String newName, int position) {
        vColNames.insertElementAt(newName, position);
    }
    /** 
      * Sets a column name in the certain position in the list. 
      * @param newName is column name.
      * @param position is insertion poistion.
      * @since   1.0           
      */
    public void setColumnName(String newName, int position) {
        if (vColNames.size() <= position)
            insertColumnName(newName, position);
        else
            vColNames.set(position, newName);
    }
    /** 
      * Sets a list of column names to be projected out.. 
      * @param newName is column name.
      * @param position is insertion poistion.
      * @since   1.0           
      */
    public void setColumnNames(String[] columns) {
        this.vColNames.removeAllElements();
        if (columns != null)
            for (int i = 0; i < columns.length; i++)
                this.vColNames.addElement(columns[i]);
    }
    /**
     * returns a string representation of column names list. 
     * @return a string representation of column names list.
     * @since   1.0          
     */
    public String toString() {
        if (vColNames.size() == 0) {
            return "";
        }
        String sOutput = vColNames.elementAt(0).toString();
        for (int i = 1; i < vColNames.size(); i++) {
            sOutput = sOutput + "," + vColNames.elementAt(i).toString();
        }
        return sOutput;
    }
    /**
     * Updates a specific column name in the list with a new name.
     * @param oldname is the old column name.
     * @param newname is new column name.
     * @since   1.0          
     */
    public void updateColumnName(String oldname, String newname) {
        // Find all the occurances of the old name and replace with new name.
        for (int i = 0; i < vColNames.size(); i++) {
            String item = (String) vColNames.elementAt(i);
            if (item.equals(oldname)) {
                vColNames.setElementAt(newname, i);
            }
        }
    }
}
