package edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;

/**
 * Implementation for interface SQLStmt .
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0      
 */

public class SQLStmtImp extends XATOperatorImp implements SQLStmt {
    protected String sSqlStmt;
    protected String sOrgColumnName;

    /**
     * Assigns a node to this operator .
     * @param xnodInit edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @since   1.0        
     */
    public SQLStmtImp(XATNode xnodInit) {
        super(xnodInit);
    }
    /**
     * Assigns a node to this operator and the sql statement.
     * @param xnodInit edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param sSqlStmtInit is teh the sql statement
     * @since   1.0        
     */
    public SQLStmtImp(XATNode xnodInit, String sSqlStmtInit) {
        super(xnodInit);
        sSqlStmt = sSqlStmtInit;
    }
    /** 
    * It used to implement a visitor pattern. 
    * @param visitor for the visitor pattern
    * @since   1.0       
    */
    public void accept(Visitor visitor) {

    }
    /**
     * Get primary data source.
     * The primary data source will be the first data source in all the operators.
     * For example:
     * For all the Join the primary data source is source l.
     * For all the Union operators is the first source.
     * For CO is the source S1.
     * @return XATTable of the primary data source.
     * @since   1.0       
     */
    /*    public XATTable getPrimarySource()
    	{
    		return xnodOwner.getChild(0).getOperator().getOutput();
    	} 
    */

    /**
     * Get the current SQL statement in the SQL operator.
     * @return current SQL language.
     * @since   1.0        
     */
    public String getStmt() {
        return sSqlStmt;
    }
    /**
     * Set the SQL stored in this SQL operator.
     * @param newSQL the new SQL statement.
     * @return original SQL statement.
     * @since   1.0        
     */
    public void setStmt(String newSQL) {
        sSqlStmt = newSQL;
    }
    /**
     * returns a string representation 
     * @return java.lang.String
     * @since   1.0       
     */
    public String toString() {
        return "SQL[" + sSqlStmt + "]";
    }
}
