package edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import edu.wpi.cs.dsrg.xmldb.xat.component.operatorexec.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import java.util.*;

/**
 * The FunCall class
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0    
*/
public abstract class FunCall {
    /**
     * if later func implement eval, it will over load this function
     * otherwise, func implement exec() method, it will over load exec function    
     * @param f edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function.FunCall
     * @since   1.0        
     */
    final public void eval(Function f) {
        iterator(f);
    }

    /**
     * The subclass should overwrite this method.
     * @param param java.util.Vector
     * @return the result of execution.          
     * @throws Exception
     * @since   1.0        
     */
    abstract protected Object exec(Vector param) throws Exception;

    /**
     * Iterates through function parameter.
     * @param f is the function.
     * @since   1.0        
     */
    public void iterator(Function f) {
        try {
            XATTable xatInput = (XATTable) f.getPrimaryDataSource();
            XATTable xatOutput = (XATTable) xatInput.deepCloneDataSource();
            int resultColNum = xatOutput.getNumberOfColumns();
            int resultTupleNum = xatOutput.getNumberOfTuples();
            // original parameters.
            Object[] params = f.getParams();
            // dereferenced paramters.
            Vector derefparamV = new Vector();
            // copied the parameters over to the deref parameters.
            for (int i = 0; i < params.length; i++)
                derefparamV.add(params[i]);

            Vector resultV = new Vector();

            //Here for Fun Call, we do not consider empty XATTable case.
            //Then the Fun Call will do nothing on empty table.
            //This case should be handled by Nest.
            for (int i = 0; i < resultTupleNum; i++) {
                // deref the column names if there is any.
                for (int j = 0; j < params.length; j++) {
                    if (params[j] instanceof ColumnName) {
                        derefparamV.setElementAt(
                            xatInput.getValue(
                                xatInput.getColumnPosition(((ColumnName) params[j]).getColumnName()),
                                i + 1),
                            j);
                    } else
                        if (params[j] instanceof XATTree) {
                            // evaluate this tree
                            // then output the result here.
                            ExecuteVisitor exec = new ExecuteVisitor();
                            // remember the result here.
                            // get the result, 
                            XATTable resultTable =
                                exec.execute((XATTree) params[j], (XATTable) f.getPrimaryDataSource());
                            XATDataValue resultValue =
                                (XATDataValue) resultTable.getValue(resultTable.getNumberOfColumns(), 1);
                            derefparamV.setElementAt(resultValue, j);
                        }
                }
                // call the function to get the output value.
                resultV.add(new XATDataValueImp(exec(derefparamV)));
            }
            xatOutput.appendColumn(f.getOutputColumnName(), resultV);
            f.setOutput(xatOutput);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}