package edu.wpi.cs.dsrg.xmldb.xat.common.operator.updateoperator;
/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;

/**
 * Implementation for the Replace interface.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0   
 */
public class ReplaceImp extends UpdateOperatorImp implements Replace {

    String PCDATAContent;
    String replacingVariable;

    /** integer indicating update function name (new_attribute() OR new_ref() )*/
    protected int updateFunction = 2;

    /** vector containing the arguments of the function */
    protected String[] funcArgs;

    /**
     * Assigns a node to this operator .
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @since   1.0   
     */
    public ReplaceImp(XATNode xnodInit) {
        super(xnodInit);
    }

    /** Replace $x WITH 
     * Assigns a node to this operator, a context, and a variable.
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param contextInit String
     * @param variableInit String     
     * @since   1.0   
     */
    public ReplaceImp(XATNode xnodInit, String contextInit, String variableInit) {
        super(xnodInit);
        context = contextInit;
        variable = variableInit;
        replacingVariable = null;
    }

    /** Replace $x WITH $y
     * Assigns a node to this operator, a context, a variable, and a replacing variable.
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param contextInit String
     * @param variableInit String
     * @param repVar String     
     * @since   1.0   
     */
    public ReplaceImp(
        XATNode xnodInit,
        String contextInit,
        String variableInit,
        String repVar) {
        super(xnodInit);
        context = contextInit;
        variable = variableInit;
        replacingVariable = repVar;
    }

    /** Replace WITH new_attribute()/new_ref() 
     * Assigns a node to this operator, a context, a variable, a replacing variable, and function arrguments.
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param contextInit String
     * @param variableInit String 
     * @param repVar String 
     * @param params an array of String       
     * @since   1.0   
     */
    public ReplaceImp(
        XATNode xnodInit,
        String contextInit,
        String variableInit,
        int updateFunctionInit,
        String[] params) {
        super(xnodInit);
        context = contextInit;
        variable = variableInit;
        updateFunction = updateFunctionInit;
        funcArgs = params;

    }

    /** 
     * Gets the PCDATA content of the operator 
     * @return int
     * @since   1.0        
     */
    public String getPCDATAContent() {
        return PCDATAContent;
    }

    /**
    * Gets the right (secondary) data source
    * @return XATTAble containing columns representing the new element
    * throws XATNodeDoesNotExistException
    * @since   1.0       
    */
    public XATTable getRight() throws XATNodeDoesNotExistException {
        try {
            return xnodOwner.getChild(1).getOperator().getOutput();
        } catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
    }

    /** 
     * Gets the update function
     * @return int
     * @since   1.0        
     */
    public int getUpdateFunction() {
        return updateFunction;
    }

    /**
     * Sets the PCDATA content that should replace the original content
     * @param String
     * @since   1.0       
     */
    public String[] getFuncArgs() {
        return funcArgs;
    }

    /**
     * Gets the replacing varible.
     * @param newVar String
     * @since   1.0       
     */
    public String getReplacingVariable() {
        return replacingVariable;
    }

    /**
     * Sets the PCDATA content that should replace the original content
     * @param funcArgs an array of String
     * @since   1.0       
     */
    public void setFuncArgs(String[] funArgs) {
        funcArgs = funArgs;
    }

    /**
    * Sets the PCDATA content that should replace the original content
    * @param String
    * @since   1.0       
    */
    public void setPCDATAContent(String pcdata) {
        PCDATAContent = pcdata;
    }

    /**
     * Sets the replacing varible.
     * @param newVar String
     * @since   1.0       
     */
    public void setReplacingVariable(String repVar) {
        replacingVariable = repVar;
    }

    /**
     * Sets the update function
     * @param funcType int
     * @since   1.0       
     */
    public void setUpdateFunction(int updateFunc) {
        updateFunction = updateFunc;
    }

}