package edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator;
/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Vector;
import java.lang.reflect.Array;

import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;

/**
 * Implementation for the Aggregate interface.
 * Aggregate treats the whole input table as one group, and apply the XATTree on the input table.
 * We distinguish the Aggregate from the GroupBy operator, because we think the Aggregate is for the XML query,
 * but the GroupBy is for the SQL query.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0   
 */
public class AggregateImp extends XATOperatorImp implements Aggregate {
    XATTree xtreFunctions;

    /**
     * Assigns a node to this operator .
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @since   1.0   
     */
    public AggregateImp(XATNode xnodInit) {
        super(xnodInit);
    }

    /**
     * Assigns a node to this operator and the XAT tree.
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param xtreFuncInit XATTree    
     * @since   1.0   
     */
    public AggregateImp(XATNode xnodInit, XATTree xtreFuncInit) {
        super(xnodInit);
        xtreFunctions = xtreFuncInit;
    }

    /** 
     * Used to implement a visitor pattern. 
     * @param visitor for the visitor pattern
     * @since   1.0       
     */
    public void accept(Visitor visitor) {

    }

    /** 
     * Gets the XAT tree associated with this operator.
     * @return XATTree
     * @since   1.0        
     */
    public XATTree getXATTree() {
        return xtreFunctions;
    }

    /** 
     * Sets the XAT tree (the group Operators) for this aggregate.     
     * @param String
     * @since   1.0       
     */
    public void setXATTree(XATTree newXAT) {
        xtreFunctions = newXAT;
    }

    /**
    * Returns a String that represents the value of this object.
    * @return a string representation of the receiver
    * @since   1.0          
    */
    public String toString() {
        return "Aggregate";
    }
}