package edu.wpi.cs.dsrg.xmldb.xat.common.xatnode;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import org.w3c.dom.Node;
import java.util.*;
import org.w3c.dom.Element;

import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;

/**
 * <pre>
 * The class that represents the value of one cell in an XATTable
 * The value is an atomic value or a Collection and 
 * can be a Dom or a String datatype.  Methods are provided that will allow
 * one type to be converted to another.  
 * </pre>
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0  
 */

public interface XATDataValue extends XATValue {
    /**
     * If you use add, means this XATDataValue is a collection.
     * This is the only method that can convert an atomic value into
     * a collection.
     * @return void
     * @param obj Add a given Object into XATDataValue.
     * @since 1.0
     */
    void add(Object obj);

    /**
     * Converts the XATValue into a Boolean object.
     * @return Boolean A boolean object after convert.
     * @since 1.0
     */
    Boolean convertToBoolean();
    
    /**
     * This method will take whatever the value is and converts it into
     * a Collection (Vector)
     * @return Vector The convert result.
     * @since 1.0
     */
     
    Vector convertToCollection();
    /**
    * Convert the XATDataValue as a DOM object, and construct as an XML Node.
    * @return Node The convert result.
    * @since 1.0
    */
    
    org.w3c.dom.Node convertToDOM();
    /**
     * Convert the XATDataValue into a Double object.
     * @return Double The convert result.
     * @since 1.0
     */
     
    Double convertToDouble();
    /**
     * Converts the XATValue into a String.  
     * If the Value is a collection, then one large string is created.
     * @return String The convert result.
     * @since 1.0
     */

    String convertToString();
    /**
     * Get the Hash code of this XATDataValue.
     * @return String The Hash code of this XATDataValue.
     * @since 1.0
     */
    String getId();
    
    /**
     * Remove the duplicated values in a vector.
     * The comparison is done by values.
     * @return void
     * @since 1.0
     */
    void removeDuplicates();
}