package edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Hashtable;
import java.util.Set;
import java.util.HashSet;

import edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup.SchemaCleanupNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.InheritableVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;

/**
 * This class is used to compute the Full Schema.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */
public class CompleteSchemaVisitor extends InheritableVisitor
{
	/**
	 * The visitor used to compute the produced columns of an operator.
	 * @since 1.0
	 */
	ProducedVisitor producedVisitor;
	/**
	 * The visitor used to compute the consumed columns of an operator.
	 * @since 1.0
	 */	
	ConsumedVisitor consumedVisitor;
/**
 * Default constructor of CompleteSchemaVisitor.
 * @since 1.0
 * @see ProducedVisitor
 * @see ConsumedVisitor
 */
public CompleteSchemaVisitor() {
    this.producedVisitor = new ProducedVisitor();
    this.consumedVisitor = new ConsumedVisitor();
}
    public Object dispatch()
    {
	return null;
    }
    public Object dispatch(Object o)
    {
	return o;
    }
/**
 * Get the value of the propertity "DEBUG_SchemaCleanUp".
 * @return boolean True: With debug information. False: No debug information.
 * @since 1.0
 */
public static boolean isDebug() {
	return System.getProperty("DEBUG_SchemaCleanUp")!=null;
}
/**
 * Constructe one element set using given object o.
 * @return Set A result Set object created.
 * @param o The object used to construct the set object.
 * @since 1.0
 */
protected Set oneElementSet(Object o) {
    Set s = new HashSet();
    s.add(o);
    return s;
}
/**
 * Compute the complete schema for each node.
 * The complete schema include all produced columns by its descendant 
 * and all consumed columns by itself.
 * @return Object The schema computed for given node.
 * @param node The XATNode computing schema for.
 * @since 1.0
 * @see XATNode
 */
public Object visit(XATNode node) {
    // get is produced.
    HashSet produced = (HashSet) producedVisitor.visit(node);
    HashSet consumed = (HashSet) consumedVisitor.visit(node);

    XATNode[] children = node.getChildren();

    HashSet schema = new HashSet(produced);
    if (consumed != null)
        schema.addAll(consumed);

    if (children.length != 0) {
        // append all the children's produced columns.
        for (int i = 0; i < children.length; i++) {
            schema.addAll((HashSet) visit(children[i]));
        }
    }

    node.setSchema(schema);
    return schema;
}
}
